@@ -24,57 +24,57 @@ what_next = function(model = NULL,
 
   # check verbose
   if (!is.logical(verbose)) {
-    cat("Warning: verbose provided mus be logical (TRUE or FALSE). Setting to False. \n")
+    message("Warning: verbose provided mus be logical (TRUE or FALSE). Setting to False.")
     verbose = FALSE
   }
   if (!is.logical(r2_diff)) {
     if (verbose)
-      cat("Warning: r2_diff provided mus be logical (TRUE or FALSE). Setting to TRUE. \n")
+      message("Warning: r2_diff provided mus be logical (TRUE or FALSE). Setting to TRUE.")
     r2_diff = TRUE
   }
 
   # check if model or model_table is provided is correct
   if (is.null(model)) {
-    cat("Error: no model provided. Returning NULL. \n")
+    message("Error: no model provided. Returning NULL.")
     return(NULL)
   } else{
     if (!is(model,class2 = 'lm')) {
-      cat("Error: model must be of type 'lm'. Returning NULL. \n")
+      message("Error: model must be of type 'lm'. Returning NULL.")
       return(NULL)
     }
     else{
       if (!('dv' %in% names(model))) {
-        cat("Error: model object must contain 'dv'. Returning NULL. \n")
+        message("Error: model object must contain 'dv'. Returning NULL.")
         return(NULL)
       } else{
         dv = model$dv
       }
 
       if (!('model_table' %in% names(model))) {
-        cat("Error: model object must contain 'model_table'. Returning NULL. \n")
+        message("Error: model object must contain 'model_table'. Returning NULL.")
         return(NULL)
       } else{
         model_table = model$model_table
       }
 
       if (!('meta_data' %in% names(model))) {
-        cat("Warning: model object does not contain 'meta_data'. \n")
+        message("Warning: model object does not contain 'meta_data'.")
       }
       meta_data = model$meta_data
 
       if (!('id_var' %in% names(model))) {
-        cat("Warning: model object does not contain 'id_var'.\n")
+        message("Warning: model object does not contain 'id_var'.")
       }
       id_var = model$id_var
 
       if (!('trans_df' %in% names(model))) {
-        cat("Warning: model object does not contain 'trans_df'. \n")
+        message("Warning: model object does not contain 'trans_df'.")
       }
       trans_df = model$trans_df
 
       if (!('normalise_by_pool' %in% names(model))) {
         if (verbose)
-          cat("Warning: model object does not contain normalise_by_pool. Setting to FALSE. \n")
+          message("Warning: model object does not contain normalise_by_pool. Setting to FALSE.")
         normalise_by_pool = FALSE
       } else{
         normalise_by_pool = model$normalise_by_pool
@@ -85,8 +85,8 @@ what_next = function(model = NULL,
 
   if (is.null(data)) {
     if (!('data' %in% names(model))) {
-      cat(
-        "Error: no data provided and model object does not contain 'data'. Returning NULL. \n"
+      message(
+        "Error: no data provided and model object does not contain 'data'. Returning NULL."
       )
       return(NULL)
     } else{
@@ -132,7 +132,7 @@ what_next = function(model = NULL,
 
   model_vars_t = model_table %>%
     # replace raw var name with var_t
-    get_variable_t() %>%
+    get_variable_t(trans_df = trans_df) %>%
     pull(variable_t)
 
   # get starting model results
@@ -154,6 +154,9 @@ what_next = function(model = NULL,
       # get model summary
       ms = summary(model)
 
+      # remove back-tick
+      rownames(ms$coefficients) = gsub(x =rownames(ms$coefficients),pattern = '`',replacement = '')
+
       # generate row
       coef = TRY(ms$coefficients[var, "Estimate"])
       if (is.null(coef)) {
@@ -249,28 +252,35 @@ get_vector_from_str = function(string, sep = ',', zero = TRUE) {
 #' @param data \code{data.frame} containing data from analysis
 #' @param r2_diff A boolean to determine whether to add a column to compare new and original model R2
 #' @param verbose A boolean to specify whether to print warnings
-#' @importFrom purrr reduce
 #' @importFrom methods is
 #' @importFrom stats lm
 #' @import dplyr
 #' @export
 #' @return \code{data.frame} mapping variables' transformations to the respective model's statistics.
 #' @examples
-#' run_model(data = mtcars,dv = 'mpg',ivs = c('disp','cyl')) %>%
-#' what_trans(variable = 'cyl',trans_df = data.frame(
-#' name = c('diminish', 'decay', 'lag', 'ma', 'log', 'hill', 'sin', 'exp'),
-#' func = c('linea::diminish(x,a)',
-#'          'linea::decay(x,a)',
-#'          'linea::lag(x,a)',
-#'          'linea::ma(x,a)',
-#'          'log(x,a)',
-#'          "linea::hill_function(x,a,b,c)",
-#'          'sin(x*a)',
-#'          '(x^a)'),order = 1:8) %>%
+#' model = run_model(data = mtcars,dv = 'mpg',ivs = c('disp','cyl'))
+#'
+#' trans_df = data.frame(
+#'      name = c('diminish', 'decay', 'lag', 'ma', 'log', 'hill', 'sin', 'exp'),
+#'      ts = c(FALSE,TRUE,TRUE,TRUE,FALSE,FALSE,FALSE,FALSE),
+#'      func = c('linea::diminish(x,a)',
+#'               'linea::decay(x,a)',
+#'               'linea::lag(x,a)',
+#'               'linea::ma(x,a)',
+#'               'log(x,a)',
+#'               "linea::hill_function(x,a,b,c)",
+#'               'sin(x*a)',
+#'               '(x^a)'),order = 1:8) %>%
 #'   dplyr::mutate(val = '') %>%
 #'   dplyr::mutate(val = dplyr::if_else(condition = name == 'hill',
 #'                                      '(1,5,50),(1 ,5,50),(1,5,50)',
-#'                                      val)))
+#'                                      val))
+#'
+#' variable = 'cyl'
+#'
+#' model %>%
+#'  what_trans(variable = variable,trans_df = trans_df)
+#'
 what_trans = function(model = NULL,
                       trans_df = NULL,
                       variable = NULL,
@@ -279,21 +289,25 @@ what_trans = function(model = NULL,
                       verbose = FALSE) {
   # checks  ####
 
+  # data = NULL
+  # r2_diff = TRUE
+  # verbose = FALSE
+
   if (!is.logical(verbose)) {
-    cat("Warning: verbose provided mus be logical (TRUE or FALSE). Setting to False. \n")
+    message("Warning: verbose provided mus be logical (TRUE or FALSE). Setting to False.")
     verbose = FALSE
   }
   if (!is.logical(r2_diff)) {
     if (verbose)
-      cat("Warning: r2_diff provided mus be logical (TRUE or FALSE). Setting to TRUE. \n")
+      message("Warning: r2_diff provided mus be logical (TRUE or FALSE). Setting to TRUE.")
     r2_diff = TRUE
   }
   if (is.null(trans_df)) {
-    cat("Error: trans_df must be provided. Returning NULL. \n")
+    message("Error: trans_df must be provided. Returning NULL.")
     return(NULL)
   }
   if (is.null(variable)) {
-    cat("Error: variable must be provided. Returning NULL. \n")
+    message("Error: variable must be provided. Returning NULL.")
     return(NULL)
   }
 
@@ -301,24 +315,24 @@ what_trans = function(model = NULL,
   # check if model or model_table is provided is correct
   if (is.null(model)) {
     if (is.null(model_table)) {
-      cat("Error: no model or model_table provided. Returning NULL. \n")
+      message("Error: no model or model_table provided. Returning NULL.")
       return(NULL)
     }
   } else{
     if (!is(model,class2 = 'lm')) {
-      cat("\n Error: model must be of type 'lm'. Returning NULL.")
+      message("Error: model must be of type 'lm'. Returning NULL.")
       return(NULL)
     }
     else{
       if (!('dv' %in% names(model))) {
-        cat("\n Error: model object must contain 'dv'. Returning NULL.")
+        message("Error: model object must contain 'dv'. Returning NULL.")
         return(NULL)
       } else{
         dv = model$dv
       }
 
       if (!('model_table' %in% names(model))) {
-        cat("\n Error: model object must contain 'model_table'. Returning NULL.")
+        message("Error: model object must contain 'model_table'. Returning NULL.")
         return(NULL)
       } else{
         model_table = model$model_table
@@ -330,7 +344,7 @@ what_trans = function(model = NULL,
   if (is.null(data)) {
     data = model$data
     if (is.null(data)) {
-      cat('Error: no data provided as data or in model. \n')
+      message('Error: no data provided as data or in model.')
     }
   }
 
@@ -370,6 +384,7 @@ what_trans = function(model = NULL,
 
   # clean trans_df
   trans_df = trans_df %>%
+    check_trans_df() %>%
     mutate(val = gsub(
       pattern = ' ',
       replacement = '',
@@ -389,7 +404,7 @@ what_trans = function(model = NULL,
   ) %>%
     zoo::na.fill('') %>%
     as.data.frame() %>%
-    reshape2::melt(id.vars = c('name', 'order', 'func'),
+    reshape2::melt(id.vars = c('name', 'order', 'func','ts'),
                    factorsAsStrings = FALSE) %>%
     filter(value != '')  %>%
     mutate(value = gsub(
@@ -416,7 +431,7 @@ what_trans = function(model = NULL,
     v = trans_df[x, ] %>%
       as.vector()
 
-    v = v[5:length(v)]
+    v = v[!(names(v) %in% c('name','order','func','ts','variable'))]
     v = as.numeric(v[v != ''])
 
     return(v)
@@ -575,8 +590,9 @@ what_trans = function(model = NULL,
 #' @param data \code{data.frame} containing data from analysis
 #' @param r2_diff A boolean to determine whether to add a column to compare new and original model R2
 #' @param dv string specifying the dependent variable name
+#' @param return_model_objects A boolean to specify whether to return model objects
 #' @param verbose A boolean to specify whether to print warnings
-#' @importFrom purrr reduce discard
+#' @importFrom purrr discard
 #' @importFrom methods is
 #' @importFrom stats lm
 #' @import dplyr
@@ -589,10 +605,10 @@ what_trans = function(model = NULL,
 #' dv = 'ecommerce'
 #' ivs = c('christmas','black.friday')
 #'
-#' model = run_model(data = data,dv = dv,ivs = ivs)
 #'
 #' trans_df = data.frame(
 #'   name = c('diminish', 'decay', 'hill', 'exp'),
+#'   ts = c(FALSE,TRUE,FALSE,FALSE),
 #'   func = c(
 #'     'linea::diminish(x,a)',
 #'     'linea::decay(x,a)',
@@ -604,21 +620,26 @@ what_trans = function(model = NULL,
 #'   dplyr::mutate(offline_media = dplyr::if_else(condition = name == 'hill',
 #'                                                '(1,50),(1),(1,100)',
 #'                                                '')) %>%
+#'   dplyr::mutate(offline_media = dplyr::if_else(condition = name == 'decay',
+#'                                               '.1,.7 ',
+#'                                               offline_media)) %>%
 #'   dplyr::mutate(online_media = dplyr::if_else(condition = name == 'decay',
 #'                                               '.1,.7 ',
 #'                                               '')) %>%
 #'   dplyr::mutate(promo = '')
 #'
-#' what_combo(model = model,trans_df = trans_df)
+#' model = run_model(data = data,dv = dv,ivs = ivs, trans_df = trans_df)
+#'
+#' combos = what_combo(model = model,trans_df = trans_df)
 #'
 #' #using the trans_df, data, and dv
 #' what_combo(trans_df = trans_df, data = data, dv = dv)
-
 what_combo = function(model = NULL,
                       trans_df = NULL,
                       data = NULL,
                       dv = NULL,
                       r2_diff = TRUE,
+                      return_model_objects = FALSE,
                       verbose = FALSE) {
 
   # TODO
@@ -629,53 +650,57 @@ what_combo = function(model = NULL,
   # checks  ####
 
   if (!is.logical(verbose)) {
-    cat("Warning: verbose provided mus be logical (TRUE or FALSE). Setting to False. \n")
+    message("Warning: verbose provided mus be logical (TRUE or FALSE). Setting to False.")
     verbose = FALSE
   }
   if (!is.logical(r2_diff)) {
     if (verbose)
-      cat("Warning: r2_diff provided mus be logical (TRUE or FALSE). Setting to TRUE. \n")
+      message("Warning: r2_diff provided mus be logical (TRUE or FALSE). Setting to TRUE.")
     r2_diff = TRUE
   }
+  if (!is.logical(return_model_objects)) {
+    if (verbose)
+      message("Warning: return_model_objects provided mus be logical (TRUE or FALSE). Setting to TRUE.")
+    return_model_objects = TRUE
+  }
   if (is.null(trans_df)) {
-    cat("Error: trans_df must be provided. Returning NULL. \n")
+    message("Error: trans_df must be provided. Returning NULL.")
     return(NULL)
   }
 
   # check if model or dv and data are provided is correct
   if (is.null(model)) {
     if (is.null(dv) | is.null(data)) {
-      cat("Error: model or dv and data must be provided. Returning NULL. \n")
+      message("Error: model or dv and data must be provided. Returning NULL.")
       return(NULL)
     } else{
+      model_null = TRUE
       vars = colnames(trans_df)
-      vars = vars[!(vars %in% c('name', 'order', 'func'))]
+      vars = vars[!(vars %in% c('name', 'order', 'func','ts'))]
       model = run_model(dv = dv,
                         data = data,
                         ivs = vars)
     }
   } else{
+    model_null = FALSE
     if (!is(model,class2 = 'lm')) {
-      cat("Error: model must be of type 'lm'. Returning NULL. \n")
+      message("Error: model must be of type 'lm'. Returning NULL.")
       return(NULL)
     }
     else{
+      dv = model$dv
       if (!is.null(dv)) {
         if (verbose) {
-          cat('Warning: replacing dv provided as argument with model dv. \n')
+          message('Warning: replacing dv provided as argument with model dv.')
         }
       }
-      if (!is.null(data)) {
-        if (verbose) {
-          cat('Warning: replacing data provided as argument with model data. \n')
-        }
+      if (is.null(data)) {
+        data = model$data
       }
     }
   }
 
-  dv = model$dv
   model_table = model$model_table
-  data = model$data
 
   # check pool
   if (model$normalise_by_pool) {
@@ -714,6 +739,7 @@ what_combo = function(model = NULL,
 
   # clean trans_df
   trans_df = trans_df %>%
+    check_trans_df() %>% 
     apply(2, function(x)
       gsub(' ', '', x)) %>%
     as.data.frame() %>%
@@ -722,9 +748,11 @@ what_combo = function(model = NULL,
 
   # get variables
   vars = colnames(trans_df)
-  vars = vars[!(vars %in% c('name', 'order', 'func'))]
-
+  vars = vars[!(vars %in% c('name', 'order', 'func','ts'))]
 
+  if(return_model_objects){
+    model_list = list()
+  }
   long_trans_df = list()
 
   for (var in vars) {
@@ -742,7 +770,7 @@ what_combo = function(model = NULL,
       zoo::na.fill('') %>%
       as.data.frame() %>%
       select(-vars[vars != var]) %>%
-      reshape2::melt(id.vars = c('name', 'order', 'func'),
+      reshape2::melt(id.vars = c('name', 'order', 'func','ts'),
                      factorsAsStrings = FALSE) %>%
       filter(value != '')  %>%
       mutate(value = gsub(
@@ -753,11 +781,11 @@ what_combo = function(model = NULL,
       rename(parameter = variable) %>%
       mutate(variable = var)
 
-    long_trans_df = rlist::list.append(long_trans_df, temp_trans_df)
+    long_trans_df = append(long_trans_df, list(temp_trans_df))
   }
 
   long_trans_df = long_trans_df %>%
-    purrr::reduce(rbind)
+    Reduce(f = rbind)
 
 
   # split each parameter (to be tested)
@@ -779,6 +807,7 @@ what_combo = function(model = NULL,
   long_combo_df = list()
 
   for (var in vars) {
+    # var = vars[1]
     temp_trans_df = long_trans_df %>%
       filter(variable == var)
 
@@ -792,7 +821,7 @@ what_combo = function(model = NULL,
       v = temp_trans_df[x, ] %>%
         as.vector()
 
-      v = v[5:(length(v) - 1)]
+      v = v[!(names(v) %in% c('name','order','func','ts','parameter','variable'))]
       v = as.numeric(v[v != ''])
 
       return(v)
@@ -804,13 +833,12 @@ what_combo = function(model = NULL,
 
     colnames(temp_trans_df)[1:length(col_names)] = col_names
 
-    long_combo_df = rlist::list.append(long_combo_df, temp_trans_df)
+    long_combo_df =  append(long_combo_df, list(temp_trans_df))
     names(long_combo_df)[length(long_combo_df)] = var
   }
 
-
   # expand.grid for all combos across variables
-  output_df = sapply(long_combo_df, function(x) {
+  output_df = lapply(long_combo_df, function(x) {
     1:nrow(x)
   }) %>%
     expand.grid()
@@ -822,19 +850,20 @@ what_combo = function(model = NULL,
     coef = 0
   ))
 
-
   # for each combo
   ## generate variable
   ## generate model
   for (i in 1:nrow(output_df)) {
-    # i = 1
-
-    # print(paste0('i - ',i))
+    # i = 2
 
     m = model$model_table
+    if(model_null){
+      m = m[0,]
+    }
 
     # for each var
     for (var in vars) {
+
       # var = vars[1]
       # print(paste0('var - ',var))
 
@@ -859,10 +888,6 @@ what_combo = function(model = NULL,
 
         f_name = fs_name[j]
 
-        var_t_name = paste0(var_t_name, '_', f_name)
-        # print(var_t_name)
-
-
         ps = temp_trans_df %>%
           filter(name == f_name) %>%
           arrange(parameter) %>%
@@ -876,25 +901,26 @@ what_combo = function(model = NULL,
         for (p in ps) {
           val = long_combo_df[[var]][output_df[i, var], paste0(f_name, '_', p)]
           vals = c(vals, val)
-          var_t_name = paste0(var_t_name, '_', val)
           # print(var_t_name)
           assign(p,val,envir = e)
         }
 
+        var_t_name = paste0(var_t_name, '_', f_name, '_', paste0(vals,collapse = ','))
 
         f = fs[j]
 
-
         if (model$normalise_by_pool) {
           for (g in groups) {
             # g=groups[1]
             x = data$temp_var[data[, pool] == g]
+            assign('x',x,envir = e)
             x = f %>% run_text(env = e)
             data$temp_var[data[, pool] == g] = x
 
           }
         } else{
           x = data$temp_var
+          assign('x',x,envir = e)
           x = f %>% run_text(env = e)
           data$temp_var = x
         }
@@ -918,10 +944,13 @@ what_combo = function(model = NULL,
     formula = build_formula(dv = model$dv, ivs = ivs_t)# run model
 
     # print(var_t_name %in% colnames(data))
-
     model_temp = lm(formula = formula,
                     data = data) %>% TRY()
 
+    if(return_model_objects){
+      model_list =  append(model_list,model_temp)
+    }
+
     # print(model_temp)
 
     # if model failed
@@ -933,6 +962,10 @@ what_combo = function(model = NULL,
       # get model summary
       ms = summary(model_temp)
 
+      # drop back ticks
+      # ...added because the var names have special characters
+      rownames(ms$coefficients) = gsub(x =rownames(ms$coefficients),pattern = '`',replacement = '')
+
       # generate row
       coef = ms$coefficients[var_t_name, "Estimate"] %>%
         TRY()
@@ -952,11 +985,10 @@ what_combo = function(model = NULL,
 
     }
   }
-  # TO OPTIMISE above
+  # TODO: OPTIMISE above
   # - do not create variables more than once (check)
   # - ...also for partial transformations (e.g. x_t1, x_t1_t2, x_t1_t3)
 
-
   if (r2_diff) {
     m0_adj_R2 = summary(model)$adj.r.squared
 
@@ -966,10 +998,137 @@ what_combo = function(model = NULL,
       select(-m0_adj_R2)
   }
 
-  output_df = output_df %>%
-    arrange(-adj_R2)
+  ordered = order(output_df$adj_R2,decreasing = T)
 
-  return(list(results = output_df,
-              trans_parameters = long_combo_df))
+  output_df = output_df[ordered,]
+
+  if(return_model_objects){
+
+    model_list = model_list[ordered]
+
+    combos = list(results = output_df,
+                  trans_parameters = long_combo_df,
+                  variables = vars,
+                  long_trans_df = long_trans_df,
+                  model_list = model_list)
+  }else{
+    combos = list(results = output_df,
+                  trans_parameters = long_combo_df,
+                  long_trans_df = long_trans_df,
+                  variables = vars)
+  }
+
+  return(combos)
 
 }
+
+#' run_combo_model
+#'
+#' generate the mode object from the output of \code{linea::what_combo()}
+#'
+#' Generate the mode object from the output of \code{linea::what_combo()}
+#' Using the specs from the output of \code{linea::what_combo()} a new model is run.
+#'
+#' @param combos output of \code{linea::what_combo()} function
+#' @param model Model object
+#' @param model_null a boolean to specify whether the model should be used as starting point
+#' @param results_row numeric value of the model (i.e. row from what_combo()$results) to run
+#' @import dplyr
+#' @export
+#' @return list of two \code{data.frame} mapping variables' transformations to the respective model's statistics.
+#' @examples
+#'
+#' # using a model object
+#' data = read_xcsv("https://raw.githubusercontent.com/paladinic/data/main/ecomm_data.csv")
+#' dv = 'ecommerce'
+#' ivs = c('christmas','black.friday')
+#'
+#'
+#' trans_df = data.frame(
+#'   name = c('diminish', 'decay', 'hill', 'exp'),
+#'   ts = c(FALSE,TRUE,FALSE,FALSE),
+#'   func = c(
+#'     'linea::diminish(x,a)',
+#'     'linea::decay(x,a)',
+#'     "linea::hill_function(x,a,b,c)",
+#'     '(x^a)'
+#'   ),
+#'   order = 1:4
+#' ) %>%
+#'   dplyr::mutate(offline_media = dplyr::if_else(condition = name == 'hill',
+#'                                                '(1,50),(1),(1,100)',
+#'                                                '')) %>%
+#'   dplyr::mutate(offline_media = dplyr::if_else(condition = name == 'decay',
+#'                                               '.1,.7 ',
+#'                                               offline_media)) %>%
+#'   dplyr::mutate(online_media = dplyr::if_else(condition = name == 'decay',
+#'                                               '.1,.7 ',
+#'                                               '')) %>%
+#'   dplyr::mutate(promo = '')
+#'
+#' model = run_model(data = data,dv = dv,ivs = ivs, trans_df = trans_df)
+#'
+#' combos = what_combo(model = model,trans_df = trans_df)
+#'
+#' combos %>%
+#'  run_combo_model(model,1)
+run_combo_model = function(combos,
+                           model,
+                           model_null = FALSE,
+                           results_row = 1){
+  res = combos$results
+  input = res[results_row, ]
+  vars = combos$variables
+  params = combos$trans_parameters
+  params_df = combos$long_trans_df
+
+  model_table = model$model_table
+
+  if(model_null){
+    model_table = model_table[0,]
+  }
+
+  # for each var
+  for (var in vars) {
+    # var = vars[1]
+
+    n = nrow(model_table) + 1
+    model_table[n, ] = ''
+    model_table[n, 'variable'] = var
+
+
+    # get the parameters selected
+    var_params = params[[var]][input[[var]], ]
+
+    # get the trans applied
+    df = params_df %>%
+      filter(variable == var) %>%
+      select(name, order, parameter)
+
+    trans = df %>%
+      pull(name) %>%
+      unique()
+
+    for (t in trans) {
+      # t = trans[2]
+
+      t_params = df %>%
+        filter(name == t) %>%
+        arrange(parameter) %>%
+        mutate(col = paste0(name, '_', parameter)) %>%
+        pull(col)
+
+      param = var_params[t_params] %>%
+        paste0(collapse = ',')
+
+      model_table[n, t] = param
+
+    }
+
+  }
+
+  model = model %>%
+    re_run_model(model_table = model_table)
+
+  return(model)
+}
